﻿package com.activetofocus.controllers
{
	import com.activetofocus.events.VideoPlayerEvent;
	import flash.events.*;
	import flash.media.SoundTransform;
	import flash.utils.Timer;
	import flash.net.NetStream;
	import flash.net.NetConnection;
	import flash.media.Video;
	
	/**
	 * VideoPlayerController
	 *
	 * @author activetofocus team
	 * @since  2010-04-03
	 */
	 
	public class VideoPlayerController extends EventDispatcher
	{
		//--------------------------------------
		//  Variables
		//--------------------------------------
		
		/**
		 *  detection interval value
		 */
		private static const DETECTION_INTERVAL:Number = 30;
		
		/**
		 *  data
		 */
		private var _autoPlay:Boolean;
		private var _isBuffering:Boolean;
		private var _isLoaded:Boolean;
		private var _volume:Number = 0.5;
		private var _isDetection:Boolean;
		
		private var _currentTime:Number;
		private var _totalTime:Number;
		
		private var _loadedBytes:Number;
		private var _totalBytes:Number;
		
		private var _detectionTimer:Timer;
		private var _doThingsArray:Array;
		
		/**
		 *  video data
		 */
		private var _video:Video;
		private var _stream:NetStream;
		private var _client:Object;
		private var _videoUrl:String;
		private var _netConnection:NetConnection;
		
		/**
    	 *  constructor
     	 */
		public function VideoPlayerController(value:Video)
		{
			_video = value;
			init();
		}
		
		//--------------------------------------
		// PRIVATE FUNCTION
		//--------------------------------------
		
		/**
		 *  data initialization
		 */
		private function init():void
		{
			_netConnection = new NetConnection();
			_netConnection.addEventListener(NetStatusEvent.NET_STATUS, netStreamStatusHandler);
            _netConnection.addEventListener(SecurityErrorEvent.SECURITY_ERROR, netStreamAsyncErrorHandler);
			_netConnection.connect(null);
			
			_detectionTimer = new Timer(DETECTION_INTERVAL);
			_detectionTimer.addEventListener(TimerEvent.TIMER,onDetectionHandler);
		}
		
		/**
		 *  start 
		 */
		 private function connectStream():void 
		 {
			_client = new Object();
			_client.onMetaData = netStreamMetaHandler;
			
			_stream = new NetStream(_netConnection);			
			_stream.client = _client;
			
			_stream.addEventListener(NetStatusEvent.NET_STATUS, netStreamStatusHandler);
			_stream.addEventListener(IOErrorEvent.IO_ERROR, netStreamIOErrorHandler);
			_stream.addEventListener(AsyncErrorEvent.ASYNC_ERROR, netStreamAsyncErrorHandler);
			_stream.soundTransform = new SoundTransform(_volume);
			
			_video.attachNetStream(_stream);
		 }
		/**
		 *  reset all data
		 */
		private function reset():void
		{
			_detectionTimer.reset();
			_doThingsArray = null;
			
			stop();
			
			_currentTime = 0;
			_totalTime   = 0;
			_loadedBytes = 0;
			_totalBytes  = 0;
			
			changeLoadProgressData();
			changeTimeUpdateData();
			changeBufferProgressData();
			
			try
			{
				_stream.close();
			}catch(e:Error){};
		}
		
		/**
		 *  change time update
		 */
		private function changeTimeUpdateData():void 
		{
			if(_stream)
			{
				var ct:Number = _stream.time;
				_currentTime = (_currentTime != ct) ? ct : _currentTime;
				_totalTime   = (_currentTime > _totalTime) ? _currentTime : _totalTime;
			}
			dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.VIDEOPLAYER_TIME_UPDATE, {currentTime:_currentTime, totalTime:_totalTime}));
			if(_stream && _currentTime == _totalTime && _totalTime != 0)
			{
				//trace("complete");
				_isDetection = false;
				dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.VIDEOPLAYER_COMPLETE));
			}
		}
		
		/**
		 *  change load progress
		 */
		private function changeLoadProgressData():void 
		{
			if(_isLoaded) return;
			if(_stream)
			{
				_loadedBytes = _stream.bytesLoaded;
				_totalBytes  = _stream.bytesTotal;
				
				if(_loadedBytes > 0 && _loadedBytes == _totalBytes)
				{
					_isLoaded = true;
					dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.BUFFER_PROGRESS,{progress:_stream.bufferLength/_stream.bufferTime}));
					if(_isBuffering)
					{
						_isBuffering = false;
						dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.BUFFER_FULL));
					}
				}
			}
			dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.LOAD_PROGRESS, {loadedBytes:_loadedBytes, totalBytes:_totalBytes}));
		}
		
		/**
		 *  change buffer progress
		 */
		private function changeBufferProgressData():void 
		{
			if(_stream)
			{
				if(_stream.bufferLength < _stream.bufferTime)
				{
					if(!_isBuffering) _isBuffering = true;
					dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.BUFFER_PROGRESS,{progress:_stream.bufferLength/_stream.bufferTime}));
				}
				else
				{
					if(_isBuffering)
					{
						_isBuffering = false;
						dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.BUFFER_FULL));
					}
				}
			}
			else
			{
				dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.BUFFER_PROGRESS,{progress:0}));
			}
		}
		
		//--------------------------------------
		// PUBLIC FUNCTION
		//--------------------------------------
		
		/**
		 *  video start load
		 */
		public function load(videoUrl:String,bufferTime :Number = 1):void
		{
			_videoUrl = videoUrl;
			
			reset();
			
			_isBuffering = true;
			_isLoaded    = false;
			_isDetection = true;
			_stream.bufferTime = bufferTime;
			_stream.play(_videoUrl);
		}
		
		/**
		 *  video play,stop,pause,resume,seek
		 */
		public function play():void
		{
			_isDetection = true;
			_stream.resume();
		}

		public function stop():void
		{
			_stream.seek(0);
			pause();
		}
		
		public function pause():void
		{
			_stream.pause();
		}
		
		public function resume():void
		{
			_stream.resume();
		}
		
		public function seek(value:Number):void
		{
			_isDetection = true;
			_stream.seek(value);
		}
		
		public function close():void
		{
			if (_stream)
			{
				_stream.pause();
				_stream.close();
			}
			if (_netConnection) _netConnection.close();
			if (_detectionTimer)
			{
				_detectionTimer.stop();
			}
		}
		
		//--------------------------------------
		// PUBLIC SET , GET 
		//--------------------------------------

		/**
		 *  auto play state
		 */
		public function set autoPlay(value:Boolean):void
		{
			_autoPlay = value;
		}
		
		public function get autoPlay():Boolean
		{
			return _autoPlay;
		}
		
		/**
		 *  volume value
		 */
		public function set volume(value:Number):void
		{
			_volume = value;
			if(_stream) _stream.soundTransform = new SoundTransform(_volume);
		}
		
		public function get volume():Number
		{
			return _volume;
		}
		
		/**
		 *  current loaded bytes
		 */
		public function get loadedBytes():Number
		{
			return _loadedBytes;
		}
		
		/**
		 *  total bytes
		 */
		public function get totalBytes():Number
		{
			return _totalBytes;
		}
		
		/**
		 *  current play time
		 */
		public function set currentTime(value:Number):void
		{
			_currentTime = value;
		}
		
		public function get currentTime():Number
		{
			return _currentTime;
		}
		
		/**
		 *  total paly time
		 */
		public function get totalTime():Number
		{
			return _totalTime;
		}

		//----------------------------------------
		//
		//----------------------------------------

		/**
		 *  detection video status timer event
		 */
		private function onDetectionHandler(evt:TimerEvent):void
		{
			if(!_isDetection) return;
			changeLoadProgressData();
			changeTimeUpdateData();
			changeBufferProgressData();
		}
		
		/**
		 *  stream client handler
		 */
		private function netStreamMetaHandler(obj:Object):void
		{
			if(obj.width && obj.height)
			{
				trace(obj.width,obj.height,_video.width,_video.height);
				_detectionTimer.start();
			}
			else
			{
				trace("netStreamMetaHandler");
			}
			if(obj.duration)
			{
				_totalTime = Number(obj.duration);
				dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.VIDEOPLAYER_TIME_UPDATE, {currentTime:_currentTime, totalTime:_totalTime}));
			}
		}
		
		/**
		 *  stream client status handler
		 */
		private function netStreamStatusHandler(evt:NetStatusEvent):void
		{
			switch (evt.info.code) {
                case "NetConnection.Connect.Success":
                    connectStream();
                    break;
                case "NetStream.Play.StreamNotFound":
                    trace("Unable to locate video: ");
					dispatchEvent(new VideoPlayerEvent(VideoPlayerEvent.VIDEO_ERROR));
                    break;
            }
		}
		
		/**
		 *  net stream io error
		 */
		private function netStreamIOErrorHandler(evt:IOErrorEvent):void
		{
			trace("netStreamIOErrorHandler");
		}
		 
		/**
		 *  net stream async error
		 */
		private function netStreamAsyncErrorHandler(evt:AsyncErrorEvent):void
		{
			trace("netStreamAsyncErrorHandler");
		}
		
	}
}