﻿package com.activetofocus.video
{
	import caurina.transitions.Tweener;
	import com.activetofocus.utils.SuperMovieClip;
	import com.activetofocus.utils.LoadXmlData;
	import com.activetofocus.events.LoadDataEvent;
	import com.activetofocus.video.events.VideoPlayerMenuComponentsEvent;
	import com.activetofocus.video.components.VideoPlayerMenuComponents;
	import com.activetofocus.controllers.VideoPlayerController;
	import com.activetofocus.events.VideoPlayerEvent;
	import com.activetofocus.utils.Clear;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.display.StageScaleMode;
	import flash.display.StageAlign;
	import flash.media.Video;
	import flash.display.StageDisplayState;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	/**
	 * Video Player
	 *
	 * @author activetofocus team
	 * @since  2010-04-12
	 */
	public class VideoMain extends SuperMovieClip
	{
		//--------------------------------------
		//  Variables
		//--------------------------------------
		
		/**
		 * data
		 */
		private var _params:Object;
		
		/**
		 *  flv data
		 */
		private var _flvPath:String;//flv path
		private var _flvDesc:String;// video information
		private var _autoplay:Boolean;// auto play state
		private var _bufferTime:Number;// video buffer time
		private var _volume:Number;// video volume
		private var _isDesc:Boolean;//desc show state
		
		/**
		 *  video controller
		 */
		private var _videoPlayController:VideoPlayerController;
		private var menu:VideoPlayerMenuComponents;
		
		/**
		 *  timer
		 */
		private var _timer:Timer;
		
		/**
    	 *  constructor
     	 */
		public function VideoMain(params:Object )
		{
			_params = params;
		}
		
		//--------------------------------------
		//  Function
		//--------------------------------------
		
		//--------------------------------------
		//  Override Function
		//--------------------------------------
		
		/**
		 * add to stage
		 */
		override public function initialize():void 
		{
			//this.stage.addEventListener(Event.RESIZE, onStageResize);
			init()
		} 
		
		/**
		 * remove from stage
		 */
		override public function destroy():void 
		{
			if(_videoPlayController) _videoPlayController.close();
			Clear.removeChild(video);
		}
		/**
		 *  start loader xml file
		 */
		public function init():void
		{
			_flvPath = _params['path'] || "assets/background/demo.flv";
			
			_autoplay = _params['auto'] == true;
			_bufferTime = 1;
			_volume = 0.5;
			
			stopBtn.addEventListener(MouseEvent.CLICK, onPlayBtnClick);
			stopBtn.buttonMode = true;
			
			menu = new VideoPlayerMenuComponents();
			menu.x = 0;
			menu.y = (_params['height'] != null && Number(_params['height']) != 340) ? Number(_params['height']) : 340;
			addChild(menu);
			
			menu.addEventListener(VideoPlayerMenuComponentsEvent.VIDEO_PLAY,onVideoPlayerMenuVideoPlay);
			menu.addEventListener(VideoPlayerMenuComponentsEvent.VIDEO_STOP,onVideoPlayerMenuVideoStop);
			menu.addEventListener(VideoPlayerMenuComponentsEvent.VIDEO_STOP_ALL,onVideoPlayerMenuVideoStopAll);
			menu.addEventListener(VideoPlayerMenuComponentsEvent.VIDEO_SEEK,onVideoPlayerMenuVideoSeek);
			menu.addEventListener(VideoPlayerMenuComponentsEvent.SOUND_CHANGE,onVideoPlayerMenuSoundChange);
			menu.addEventListener(VideoPlayerMenuComponentsEvent.FULLSCREEN_CHANGE,onVideoPlayerMenuFullscreenChange);
						
			_videoPlayController = new VideoPlayerController(video);
			_videoPlayController.addEventListener(VideoPlayerEvent.VIDEO_ERROR, onVideoPlayerVideoError);
			_videoPlayController.addEventListener(VideoPlayerEvent.LOAD_PROGRESS, onVideoPlayerLoadHandler);
			_videoPlayController.addEventListener(VideoPlayerEvent.VIDEOPLAYER_TIME_UPDATE, onVideoPlayerTimeUpdateHandler);
			_videoPlayController.addEventListener(VideoPlayerEvent.BUFFER_PROGRESS, onVideoPlayerBufferProgressHandler);
			_videoPlayController.addEventListener(VideoPlayerEvent.BUFFER_FULL, onVideoPlayerBufferFullHandler);
			_videoPlayController.addEventListener(VideoPlayerEvent.VIDEOPLAYER_COMPLETE, onVideoPlayerPlayComplete);
			
			_videoPlayController.volume = _volume;
			_videoPlayController.load(_flvPath,_bufferTime);
			if (!_autoplay) _videoPlayController.stop();
			
			stopBtn.visible = !_autoplay;
			
			menu.changePlayState(_autoplay);
			menu.changeSoundValue(Math.floor(_volume*10));
		}
		
		/**
		 *  start timer
		 */
		private function startTimer():void
		{
			stopTimer();
			_timer = new Timer(2500,1);
			_timer.addEventListener(TimerEvent.TIMER_COMPLETE,onTimerComplete);
			_timer.start();
		}
		
		/**
		 *  stop timer
		 */
		private function stopTimer():void
		{
			if(_timer)
			{
				_timer.stop();
				_timer.removeEventListener(TimerEvent.TIMER_COMPLETE,onTimerComplete);
				_timer = null;
			}
		}
		
		
		//--------------------------------------
		//  Event
		//--------------------------------------
		
		/**
		 *  video load error
		 */
		private function onVideoPlayerVideoError(evt:VideoPlayerEvent):void
		{
			
		}
		
		/**
		 *  timer complete
		 */
		private function onTimerComplete(evt:TimerEvent):void
		{
			_timer.removeEventListener(TimerEvent.TIMER_COMPLETE,onTimerComplete);
			Tweener.addTween(menu,{time:0.5,y:this.stage.stageHeight+40});
		}
				
		private function onVideoPlayerLoadHandler(evt:VideoPlayerEvent):void
		{
			//trace("videoplayer load progress: " + evt.data.loadedBytes);
			menu.changeLoadProgress(Number(evt.data.loadedBytes/evt.data.totalBytes));
		}
		
		private function onVideoPlayerTimeUpdateHandler(evt:VideoPlayerEvent):void
		{
			//trace("videoplayer time update: " + evt.data.currentTime);
			menu.changePlayProgress(evt.data.currentTime,evt.data.totalTime);
		}
		
		private function onVideoPlayerBufferProgressHandler(evt:VideoPlayerEvent):void
		{
			var k:int = Math.floor(evt.data.progress * 100);
			k = k>=100 ? 100 : k;
			prog.txt_mc.text = k.toString();
			prog.visible = true;
			prog.play();
		}
		
		/**
		 *  video play buffer full
		 */
		private function onVideoPlayerBufferFullHandler(evt:VideoPlayerEvent):void
		{
			prog.visible = false;
			prog.gotoAndStop(1);
		}
		
		/**
		 *  video play complete
		 */
		private function onVideoPlayerPlayComplete(evt:VideoPlayerEvent):void
		{
			//trace("play complete");
			_videoPlayController.seek(0);
			_videoPlayController.pause();
			menu.changePlayState(false);
			stopBtn.visible = true;
		}
		
		/**
		 *  video play
		 */
		private function onVideoPlayerMenuVideoPlay(evt:VideoPlayerMenuComponentsEvent):void
		{
			stopBtn.visible = false;
			menu.changePlayState(true);
			_videoPlayController.play();
		}
		
		/**
		 *  video stop
		 */
		private function onVideoPlayerMenuVideoStop(evt:VideoPlayerMenuComponentsEvent):void
		{
			if(evt.data == null) menu.changePlayState(false);
			_videoPlayController.pause();
			stopBtn.visible = true;
		}
		
		/**
		 *  video stop all
		 */
		private function onVideoPlayerMenuVideoStopAll(evt:VideoPlayerMenuComponentsEvent):void
		{
			stopBtn.visible = true;
			if(evt.data == null) menu.changePlayState(false);
			_videoPlayController.seek(0);
			_videoPlayController.pause();
		}
		
		/**
		 *  video seek
		 */
		private function onVideoPlayerMenuVideoSeek(evt:VideoPlayerMenuComponentsEvent):void
		{
			_videoPlayController.seek(Number(evt.data * _videoPlayController.totalTime));
		}
		
		/**
		 * sound change
		 */
		private function onVideoPlayerMenuSoundChange(evt:VideoPlayerMenuComponentsEvent):void
		{
			menu.changeSoundValue(evt.data.volume);
			_videoPlayController.volume = Number(evt.data.volume/10);
		}
		
		/**
		 *  full change
		 */
		private function onVideoPlayerMenuFullscreenChange(evt:VideoPlayerMenuComponentsEvent):void
		{
			if(stage.displayState == StageDisplayState.FULL_SCREEN)
			{
				stage.displayState=StageDisplayState.NORMAL;
			}
			else
			{
				stage.displayState=StageDisplayState.FULL_SCREEN;
			}
		}
		
		/**
		 *  stage resize
		 */
		private function onResizeEvent(evt:Event):void
		{
			
		}
		
		/**
		 *  hide/show menu
		 */
		private function onStageMouseMove(evt:MouseEvent):void
		{
			if(menu.y != this.stage.stageHeight-30) Tweener.addTween(menu,{time:0.4,y:this.stage.stageHeight-30});
			startTimer();
		}
		
		/**
		 * 
		 */
		private function onPlayBtnClick(evt:MouseEvent):void
		{
			onVideoPlayerMenuVideoPlay(null);
		}
		
	}
}